#! /usr/bin/perl -w -I ..
#
# check_perfmon tests
#
# This test requires the check_perfmon_logdir.tar.bz2 taball, a
# collection to csv files especially crafted to stress-test the
# parser library.
#
#	Still missing: MAX_AGE tests, perfdata tests
#

use strict;
use Test::More;
use NPTest;

my $successOutput = '/^PERFMON OK: /';
my $warningOutput = '/^PERFMON WARNING: /';
my $failureOutput = '/^PERFMON CRITICAL: /';
my $unknownOutput = '/^PERFMON UNKNOWN: /';

# Should likely be parametrable the usual way...
my $logdir = "t/check_perfmon_logdir";

# Try to make sure we have a complete set
if (-d $logdir && -f "$logdir/SERVER.csv" || -f "$logdir/SERVERLONG.csv"
    && -f "$logdir/BROKEN0.csv" && -f "$logdir/BROKEN4505.csv") {
	plan tests => 4 + 12 + 12 + (4506 * 2);
} else {
	plan skip_all => "Missing $logdir or files in $logdir";
}

my $result;



# Check list function (4 tests)
$result = NPTest->testCmd(
	"./check_perfmon -H SERVER -L $logdir -C list"
);
cmp_ok( $result->return_code, '==', 0, "List returns OK" );
cmp_ok( scalar(@{[split(/\n/, $result->output)]}), '==', 26, "List returns 26 lines" );

$result = NPTest->testCmd(
	"./check_perfmon -H SERVERLONG -L $logdir -C list"
);
cmp_ok( $result->return_code, '==', 0, "Long list returns OK" );
cmp_ok( scalar(@{[split(/\n/, scalar($result->output))]}), '==', 251, "Long list returns 251 lines" );



# Various test cases (12 tests)
$result = NPTest->testCmd(
	"./check_perfmon -H nosuchhost -L $logdir -C '\\Memory\\Committed Bytes'"
);
cmp_ok( $result->return_code, '==', 3, "Unknown if file is missing" );
is( $result->output, "PERFMON UNKNOWN: Error opening log file: No such file or directory" );

$result = NPTest->testCmd(
	"./check_perfmon -H SERVERLONG -L $logdir -C '\\no\\such counter'"
);
cmp_ok( $result->return_code, '==', 3, "Unknown if counter is missing" );
is( $result->output, "PERFMON UNKNOWN: No matching index '^no^such counter'", "Known result - unknown counter" );

$result = NPTest->testCmd(
	"./check_perfmon -H SERVER -L $logdir -C '\\Memory\\Committed Bytes' -w 1218310143 -c 1218310144"
);
cmp_ok( $result->return_code, '==', 1, "Critical is exclusive" );
is( $result->output, 'PERFMON WARNING: ^Memory^Committed Bytes: 1218310144| counter=1218310144.000000c;1218310143.000000;1218310144.000000;', "Larger value" );

# Header only
$result = NPTest->testCmd(
	"./check_perfmon -H BROKEN1198 -L $logdir -C '\\Memory\\Committed Bytes' -w 1218310143 -c 1218310144"
);
cmp_ok( $result->return_code, '==', 3, "Header only returns UNKNOWN" );
is( $result->output, 'PERFMON UNKNOWN: Failed to read last log line', "Header only known result" );

# Missing index
$result = NPTest->testCmd(
	"./check_perfmon -H SHORTLINE -L $logdir -C '\\Memory\\Committed Bytes'"
);
cmp_ok( $result->return_code, '==', 3, "Missing index returns UNKNOWN" );
is( $result->output, 'PERFMON UNKNOWN: Column 14 not found', "Known Missing index result" );

# Very small file
$result = NPTest->testCmd(
	"./check_perfmon -H SHORT -L $logdir -C '\\ASP.NET Applications(__Total__)\\Request Execution Time'"
);
cmp_ok( $result->return_code, '==', 0, "Short file works" );
is( $result->output, 'PERFMON OK: ^ASP.NET Applications(__Total__)^Request Execution Time: 15| counter=15.000000c;;;', "Know Short file result" );


# Check various counters (2x 6 tests)
foreach my $srv ('SERVER', 'SERVERLONG') {
	$result = NPTest->testCmd(
		"./check_perfmon -H $srv -L $logdir -C '\\Paging File(_Total)\\% Usage'"
	);
	cmp_ok( $result->return_code, '==', 0, "No thresholds always return OK ($srv)" );
	is( $result->output, 'PERFMON OK: ^Paging File(_Total)^% Usage: 1.45| counter=1.453102c;;;', "Known OK result ($srv)" );

	$result = NPTest->testCmd(
		"./check_perfmon -H $srv -L $logdir -C '\\Paging File(_Total)\\% Usage' -w 1 -c 2"
	);
	cmp_ok( $result->return_code, '==', 1, "Hit Warning threshold ($srv)" );
	is( $result->output, 'PERFMON WARNING: ^Paging File(_Total)^% Usage: 1.45| counter=1.453102c;1.000000;2.000000;', "Known WARNING result ($srv)" );

	$result = NPTest->testCmd(
		"./check_perfmon -H $srv -L $logdir -C '\\Web Service(_Total)\\Current Connections' -w 20 -c 40"
	);
	cmp_ok( $result->return_code, '==', 2, "Hit Warning threshold ($srv)" );
	is( $result->output, 'PERFMON CRITICAL: ^Web Service(_Total)^Current Connections: 44| counter=44.000000c;20.000000;40.000000;', "Known CRITICAL result ($srv)" );
}


# These tests try to catch any parsing segfault on truncated files
for (my $i = 0; $i <= 4505; $i++) {
	my ($listrc, $counterrc);
	if ($i <= 1390) {
		$listrc = 3;
	} else {
		$listrc = 0;
	}
	if ($i <= 1738) {
		$counterrc = 3;
	} else {
		$counterrc = 0;
	}
	$result = NPTest->testCmd(
		"./check_perfmon -H BROKEN$i -L $logdir -C list"
	);
	cmp_ok( $result->return_code, '==', $listrc, "Automated list $i" );

	$result = NPTest->testCmd(
		"./check_perfmon -H BROKEN$i -L $logdir -C  '\\Web Service(_Total)\\Post Requests/sec'"
	);
	cmp_ok( $result->return_code, '==', $counterrc, "Automated counter $i" );
}

